/*
 * Copyright 2015 Luca Beltrame <lbeltrame@kde.org>
 *
 * This file is part of Danbooru Client.
 *
 * Danbooru Client is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Danbooru Client is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Danbooru Client. If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "test_danbooru_post.h"
#include "../danboorupost.h"
#include "../utils.h"
#include "../servicebase.h"

#include <QFile>
#include <QJsonDocument>
#include <QJsonObject>



void TestDanbooruPost::initTestCase()
{

}

void TestDanbooruPost::testPostLoading()
{
    QFETCH(QString, jsonFile);
    auto fileName = QFINDTESTDATA(jsonFile);

    QFile file;
    file.setFileName(fileName);
    file.open(QIODevice::ReadOnly | QIODevice::Text);

    bool ok;
    auto data = Danbooru::parseDanbooruResult(file.readAll(), &ok).toList().at(0).toMap();

    if (!ok) {
        QFAIL("JSON file failed to load");
    }

    QFETCH(Danbooru::ApiType, apiType);
    QFETCH(QUrl, boardUrl);

    auto post = new Danbooru::DanbooruPost(data, apiType);
    Danbooru::fixPostUrl(post, boardUrl);

    QFETCH(int, postId);
    QFETCH(int, postWidth);
    QFETCH(int, postHeight);
    QFETCH(int, fileSize);
    QFETCH(QSet<QString>, postTags);
    QFETCH(QUrl, fileUrl);
    QFETCH(QUrl, thumbnailUrl);
    QFETCH(QUrl, sampleUrl);
    QFETCH(Danbooru::DanbooruPost::Rating, rating);

    QCOMPARE(post->id(), postId);
    QCOMPARE(post->width(), postWidth);
    QCOMPARE(post->height(), postHeight);
    QCOMPARE(post->size(), fileSize);
    QCOMPARE(post->tags(), postTags);
    QCOMPARE(post->fileUrl(), fileUrl);
    QCOMPARE(post->thumbnailUrl(), thumbnailUrl);
    QCOMPARE(post->sampleUrl(), sampleUrl);
    QCOMPARE(post->rating(), rating);
    QCOMPARE(post->pixmap().isNull(), true);

}

void TestDanbooruPost::testPostLoading_data()
{
    //TODO: Add API type
    QTest::addColumn<QString>("jsonFile");
    QTest::addColumn<Danbooru::ApiType>("apiType");
    QTest::addColumn<int>("postId");
    QTest::addColumn<int>("postWidth");
    QTest::addColumn<int>("postHeight");
    QTest::addColumn<int>("fileSize");
    QTest::addColumn<QSet< QString >>("postTags");
    QTest::addColumn<QUrl>("fileUrl");
    QTest::addColumn<QUrl>("thumbnailUrl");
    QTest::addColumn<QUrl>("sampleUrl");
    QTest::addColumn<Danbooru::DanbooruPost::Rating>("rating");
    QTest::addColumn<QUrl>("boardUrl");

    // Konachan.com / yande.re
    QTest::newRow("konachan") << "konachan.json" // jsonFile
                              << Danbooru::ApiType::KonachanDanbooru
                              << 205621     // id
                              << 1280       // width
                              << 720        // height
                              << 885610     // file_size
                              << QSet<QString>({"azuki_(sayori)", "coconut_(sayori)", "game_cg", "nekopara",
                                                "sayori"})     // tags
                              << QUrl("http://konachan.net/image/eb36c568af4f4e4ea2f59eda7d8c802d/Konachan.com%20-%20205621%20azuki_%28sayori%29%20coconut_%28sayori%29%20game_cg%20nekopara%20sayori.png")
                              << QUrl("http://konachan.net/data/preview/eb/36/eb36c568af4f4e4ea2f59eda7d8c802d.jpg")
                              << QUrl("http://konachan.net/jpeg/eb36c568af4f4e4ea2f59eda7d8c802d/Konachan.com%20-%20205621%20azuki_%28sayori%29%20coconut_%28sayori%29%20game_cg%20nekopara%20sayori.jpg")
                              << Danbooru::DanbooruPost::Safe
                              << QUrl("http://konachan.com");
    // Danbooru

    QTest::newRow("danbooru") << "donmai.json"
                              << Danbooru::ApiType::OriginalDanbooru
                              << 2106872
                              << 1200
                              << 993
                              << 514370
                              << QSet<QString>({"1girl", "all_fours", "bare_legs",
                                                "barefoot", "brown_eyes", "brown_hair", "chain",
                                                "collar", "hat", "hecatia_lapislazuli", "long_hair",
                                                "looking_at_viewer", "off_shoulder",
                                                "shirt", "skirt", "solo", "touhou",
                                                "uranaishi_(miraura)" })
                              << QUrl("http://danbooru.donmai.us/data/7ee70315eba5ea4fbda7cd6011c8d833.jpg")
                              << QUrl("http://danbooru.donmai.us/data/preview/7ee70315eba5ea4fbda7cd6011c8d833.jpg")
                              << QUrl("http://danbooru.donmai.us/data/sample/sample-7ee70315eba5ea4fbda7cd6011c8d833.jpg")
                              << Danbooru::DanbooruPost::Safe
                              << QUrl("http://danbooru.donmai.us");

    // Gelbooru
}


QTEST_MAIN(TestDanbooruPost)

